// angular.module is a global place for creating, registering and retrieving Angular modules
// 'experticity' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'experticity.controllers' is found in controllers.js
var app = angular.module('experticity',
  ['ionic', 'ionic.service.core', 'ionic.service.analytics', 'ionic.contrib.frost', 'ngIOS9UIWebViewPatch', 'ngCordova',
  'experticity.controllers', 'experticity.services', 'experticity.directives', 'experticity.filters',]);

app.run(function($ionicPlatform, $cordovaDevice, $rootScope, $state, $cordovaGoogleAnalytics, $ionicAnalytics, UserService, PagesService) {
  $rootScope.$on('$stateChangeStart',
    function (e, toState, toParams, fromState, fromParams) {
    /**
     * Chaging state monitor for if we should redirect to registration.
     */
    if (toState.name === "app.user_account") {
      UserService.profile().then(function (data) {
        if (data.profile === null || (data.profile || {}).id === null) {
          $state.go('app.user_registration', {pageFrom: "account"});
        }
      });
    }
  });

  PagesService.downloadData();

  $ionicPlatform.ready(function() {
    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    // if (window.cordova && window.cordova.plugins.Keyboard) {
    //   cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    // }
    $ionicAnalytics.register();

    // Wrap these in here so that they don't get an undefined
    // error when on a desktop.
    if (window.cordova) {
      console.log("On a device");

      console.log("App Stats", window.analytics);
      if (typeof window.analytics !== 'undefined'){
        // $cordovaGoogleAnalytics.debugMode();
        // $cordovaGoogleAnalytics.startTrackerWithId('UA-61518716-1');
        window.analytics.debugMode();
        window.analytics.startTrackerWithId('UA-61518716-1');
        console.log("Google Analytics Loaded");
      } else {
          console.log("Google Analytics Unavailable");
      }

      window.plugin.notification.local.onadd = function (id, state, json) {
        var notification = {
            id: id,
            state: state,
            json: json
        };
        $timeout(function() {
            $rootScope.$broadcast("$cordovaLocalNotification:added", notification);
        });
        $ionicPlatform.ready(function() {
            if($cordovaDevice.platform === "iOS") {
              window.plugin.notification.local.promptForPermission();
            }
        });
      };
    }
  });
});

app.config(function($ionicConfigProvider, $stateProvider, $urlRouterProvider, $ionicAppProvider) {
  $ionicConfigProvider.navBar.alignTitle('center');
  $ionicConfigProvider.views.swipeBackEnabled(false);
  $ionicConfigProvider.tabs.position('bottom');
  $ionicAppProvider.identify({
    app_id: '1c222792',
    api_key: 'f744b564abc53303b8d0fd7aa17dc4638e13ace0ede52054'
  });

  $stateProvider

  .state('app', {
    url: "/app",
    abstract: true,
    templateUrl: "templates/base_view.html",
    controller: 'AppCtrl',
    resolve: {
      resolvedUser: resolvers.resolveUser,
    }
  })

  .state('app.about', {
    url: "/about",
    views: {
      'event-tab': {
        templateUrl: "templates/info/about_the_app.html",
        controller: 'AboutTheAppCtrl',
      }
    },
    onExit: function ($ionicHistory) {
      $ionicHistory.nextViewOptions({disableBack:true, historyRoot:true});
    }
  })

  // Events
  .state('app.events', {
    url: "/events",
    views: {
      'event-tab': {
        templateUrl: "templates/events.html",
        controller: 'EventsCtrl'
      }
    },
  })

  .state('app.event_quizzes', {
    url: "/events/:eventId/quizzes/:tab?",
    views: {
      'event-tab': {
        templateUrl: "templates/event_quizzes.html",
        controller: 'EventQuizzesCtrl'
      }
    }
  })
  // End Events


  // Play State
  // Keep the history inmind to prevent users from exiting the quiz.
  // http://ionicframework.com/docs/api/service/$ionicHistory/

  // This doesn't inherit from app. Since it shouldn't have the side menu.
  .state('quiz_question', {
    // Disable caching for this view, it should always be freshly loaded.
    cache: false,
    url: "/app/quizzes/:quizId/:questionId/playing",
    templateUrl: "templates/quiz/question.html",
    controller: 'QuizQuestionCtrl',
    resolve: {
      resolvedStatus: function (PlayService) {
        return PlayService.quizStatus();
      },
    },
    onExit: clearHistoryDisableBack,
  })

  .state('quiz_double_down', {
      cache: false,
      url: "/app/quizzes/:quizId/double_down",
      templateUrl: "templates/quiz/want_to_double_down.html",
      controller: "QuizDoubleDownCtrl",
      resolve: {
        resolvedStatus: function (PlayService) {
          return PlayService.quizStatus();
        },
      },
      onExit: clearHistoryDisableBack,
  })

  .state('quiz_end', {
    cache: false,
    url: "/quizzes/:quizId/end",
    templateUrl: "templates/quiz/end.html",
    controller: 'QuizEndCtrl',
    resolve: {
      resolvedUser: resolvers.resolveUser,
      resolvedGameData: function (PlayService) {
        return PlayService.finalQuizData();
      }
    },
    onExit: clearHistoryDisableBack,
  })
  // End PlayState

  // User Info

  .state('app.user_account', {
    url: "/user/account",
    views: {
      'account-tab': {
        templateUrl: "templates/user/account.html",
        controller: "UserAccountCtrl",
      }
    },
    resolve: {
      resolvedUser: resolvers.resolveUser,
    }
  })

  .state('app.user_edit_account', {
    cache: false,
    url: "/user/account/edit",
    views: {
      'account-tab': {
        templateUrl: "templates/user/edit_account.html",
        controller: "UserEditAccountCtrl",
      }
    }
  })

  .state('app.user_signin', {
    cache: false,
    url: "/user/signin/:type",
    views: {
      'account-tab': {
        templateUrl: "templates/user/signin.html",
        controller: "UserSignInCtrl",
      }
    }
  })

  .state('app.user_forgot_password', {
    cache: false,
    url: "/user/forgot",
    views: {
      'account-tab': {
        templateUrl: "templates/user/forgot_password.html",
        controller: "UserForgotPasswordCtrl",
      }
    }
  })

  .state('app.user_registration', {
    url: "/user/registration/:pageFrom",
    views: {
      'account-tab': {
        templateUrl: "templates/user/registration.html",
        controller: "UserRegistrationCtrl",
      }
    }
  })

  // Confirm signup after a quiz. Display the code.
  .state('app.user_registration_thanks_quiz', {
    cache: false,
    url: "/user/registration/thanks/quiz",
    views: {
      'account-tab': {
        templateUrl: "templates/user/thanks_from_quiz.html",
        controller: "UserRegistrationThanksFromQuizCtrl",
      }
    },
     resolve: {
      resolvedGameData: function (PlayService) {
        return PlayService.finalQuizData();
      },
      resolvedUser: resolvers.resolveUser,
    },
    onExit: clearHistoryDisableBack
  })

  // The less fancy version if they just signed up.
  .state('app.user_registration_thanks_account', {
    cache: false,
    url:"/user/registration/thanks/account",
    views: {
      'account-tab': {
        templateUrl: "templates/user/thanks_from_account.html",
        controller: "UserRegistrationThanksFromQuizCtrl",
      }
    },
    resolve: {
      resolvedUser: resolvers.resolveUser,
      resolvedGameData: function () { return null; },
    },
    onExit: clearHistoryDisableBack,
  })

  .state('app.user_codes', {
    url: "/user/codes",
    views: {
      'prize-tab': {
        templateUrl: "templates/user/codes.html",
        controller: "UserCodesCtrl",
      }
    }
  })

  // End User Info

  // Misc Pages
  .state('app.more_info', {
    cache: false,
    url: "/more_info",
    views: {
      'info-tab': {
        templateUrl: "templates/more_info.html",
        controller: "MoreInfoCtrl",
      }
    }
  })
  .state('app.more_info_details', {
    cache: false,
    url: "/more_info/details/:pageSlug",
    views: {
      'info-tab': {
        templateUrl: "templates/info/more_info_details.html",
        controller: "MoreInfoDetailCtrl",
      }
    }
  })

  .state('fake_double_down', {
    cache: false,
    url: "/app/user/fake_double_down",
    templateUrl: "templates/quiz/want_to_double_down.html",
    controller: function ($scope) {
      $scope.quiz = {name:"Fake Quiz"};
    }
  })

  .state('fake_quiz_end', {
    cache: false,
    url: "/app/user/fake_quiz_end",
    templateUrl: "templates/quiz/end.html",
    controller: function ($scope) {
      $scope.game_data = {
        name:"Fake Quiz",
        canGetPrize: function () { return true; },
        quiz: {
          prize: "A Fake Hat",
          name: "Fakest Quiz",
        },
        event: {
          id: 3,
          grand_prize: "A giant fake Hat",
        },
        getPointsEarned: function () { return 851; },
        getCorrectAnswers: function () { return ["1", "2", "3"]; },
        getAnswers: function () { return ["1", "2", "3", "4"]; },
        getFormmatedPlayTime: function () { return "5:27"; },
      };
    }
  });

  // if none of the above states are matched, use this as the fallback
  $urlRouterProvider.otherwise('/app/events');
});

/**
 * Resolvers that are used more than once.
 */
resolvers = {
  resolveUser: function(UserService) {
    return UserService.profile();
  }
};

var clearHistoryDisableBack = function ($ionicHistory) {
  $ionicHistory.clearHistory();
  $ionicHistory.nextViewOptions({disableBack:true});
};
