var quiz_controllers_module = angular.module('experticity.quizControllers', []);

/**
 * The logic for quiz start through the end of a quiz.
 *
 */
quiz_controllers_module.controller('QuizQuestionCtrl',
  function($rootScope, $scope, $stateParams, $state, $timeout,
    resolvedStatus, PlayService) {

  console.info("QuizQuestionCtrl controller started");
  $scope.pop_quiz = true;
  $scope.show_mask = false;

  var elapsed_time = 0;
  $scope.timerTick = function (et) {
    elapsed_time = et;
  };

  $scope.choice_clicked = function (choice) {
    // Alert the directive this has been answered.
    $rootScope.$broadcast('question-answered');

    var is_correct = PlayService.answerQuestion(choice, elapsed_time, $scope.in_double_down);
    var last_question = PlayService.quizStatus().quiz_is_over;
      // Check to see if PlayService.answerQuestion returned true or false based on choice
      $scope.isCorrect = function(id) {
          if (id == $scope.question.correct_choice.id) {
              return 'correct';
          } else {
              return 'incorrect';
          }
      };

    if (is_correct) {
      $scope.choice_result = "correct";
      $scope.show_mask = true;
    } else {
      $scope.choice_result = "incorrect";
      $scope.show_mask = true;
    }

    // ANSWER_TIMEOUT is the amount of time that it will
    // wait after showing correct or incorrect.
    var ANSWER_TIMEOUT = 1 * 1000;
    if (last_question) {
      $timeout(function () {
        // No more questions. Go to the end.
        var not_in_double_down = ($scope.in_double_down === false);
        var above_double_down_thresh = (PlayService.percentageCorrect() >= $scope.quiz.double_down_prize_threshold);

        if ($scope.quiz.able_to_double_down && not_in_double_down && above_double_down_thresh) {
          $state.go("quiz_double_down", {"quizId":$scope.quiz.id});
        } else {
          PlayService.endQuiz().then(function (data) {
            $state.go("quiz_end", {"quizId":$scope.quiz.id});
          });
        }
      }, ANSWER_TIMEOUT);
    } else {
      $timeout(function () {
        var next_question = parseInt($stateParams.questionId);
        $state.go("quiz_question", { "quizId":$scope.quiz.id, "questionId":next_question+1 });
      }, ANSWER_TIMEOUT);
    }
  };

  $scope.status = resolvedStatus;
  if ($stateParams.questionId != "dd") {
    // Not a double down question
    $scope.in_double_down = false;

    $scope.question = resolvedStatus.current_question;
    $scope.question.choices = $scope.question.getChoices();
  } else {
    // A double down question
    $scope.in_double_down = true;

    $scope.question = resolvedStatus.quiz.double_down_question;
    PlayService.setDoubleDownQuestion($scope.question);
    $scope.question.choices = resolvedStatus.quiz.double_down_question.getChoices();
  }

  $scope.quiz = resolvedStatus.quiz;
});

/**
 * Double down things
 */
quiz_controllers_module.controller('QuizDoubleDownCtrl',
  function ($scope, $state, PlayService, PagesService, resolvedStatus) {

  PagesService.getAppText().then(
    function (app_text) {
      $scope.double_down_tag_line = app_text.double_down_tag_line;
      $scope.double_down_text = app_text.double_down_text;
    }
  );

  console.log("QuizDoubleDownCtrl");
  $scope.doubleDown = function () {
    $state.go("quiz_question", {"quizId":$scope.quiz.id, "questionId":"dd"});
  };

  $scope.dontDoubleDown = function () {
    PlayService.endQuiz().then(function (data) {
      $state.go("quiz_end", {"quizId": $scope.quiz.id});
    });
  };

  $scope.status = resolvedStatus;
  $scope.quiz = resolvedStatus.quiz;
});

/**
 * This quiz is over!
 */
quiz_controllers_module.controller('QuizEndCtrl',
  function($scope, resolvedUser, resolvedGameData) {

  $scope.user = resolvedUser;
  $scope.game_data = resolvedGameData;
  console.log("Game Data", $scope.game_data);
});
/**
 * The end of the quiz logic.
 */
