var user_controllers_module = angular.module('experticity.userControllers', []);

user_controllers_module.controller('UserAccountCtrl',
  function($scope, $state, $ionicHistory, UserService, resolvedUser, $ionicPlatform) {
    $ionicPlatform.ready(function() {
      if(typeof window.analytics !== 'undefined') { window.analytics.trackView("Account");
      }
    });
  $scope.user = resolvedUser.profile;

  $scope.$on('$ionicView.enter', function () {
    sync_profile();
  });

  var sync_profile = function () {
    UserService.syncProfile().then(function () {
      UserService.profile().then(function (data) {
        console.log("User Data:", data.profile);
        $scope.user = data.profile;
        $scope.stats = {
          taken_start: 0,
          taken: data.profile.contestant.stats.total_quizzes,
          points_start: data.profile.contestant.stats.points < 50000 ? data.profile.contestant.stats.points : data.profile.contestant.stats.points/2,
          points: data.profile.contestant.stats.points,
        };
      });
    });
  };
  sync_profile();
});

//$//$//$//$//$//$//$//$//$//$//$//$//$//$
// Edit Controller                      //
//$//$//$//$//$//$//$//$//$//$//$//$//$//$

user_controllers_module.controller('UserEditAccountCtrl',
  function($scope, $state, $ionicPlatform, UserService) {

  $ionicPlatform.ready(function() {
    if(typeof window.analytics !== 'undefined') { window.analytics.trackView("Account Edit");
    }
  });
  UserService.profile().then(
    function (profile) {
      $scope.user = JSON.parse(JSON.stringify(profile.profile));
      $scope.editForm = profile.profile;
    }
  );

  $scope.saveInformation = function (formData) {
    /**
     * Sets any invalid fields as told by the server as
     * invalid on the form.
     */
    var set_error_fields = function (server_response) {
      if (server_response.email) {
        $scope.form.email.$invalid = true;
      }

      if (server_response.leaderboard_name) {
        $scope.form.leaderboard_name.$invalid = true;
      }
    };
    /**
     * End Private Methods
     */

    if ($scope.form.$invalid) {
      $scope.field_errors = error_text($scope.form);
      return;
    }

    // Send the data.
    $scope.$emit('show-spinner');
    var formDataCopy = JSON.parse(JSON.stringify(formData));
    UserService.profile(formDataCopy).then(
      function (success) {
      $scope.$emit('hide-spinner');
      $state.go('app.user_account');
    }, function (failure) {
      $scope.$emit('hide-spinner');
      $scope.field_errors = error_text($scope.form, failure.errors);
      set_error_fields(failure.errors);
    });
  };
});

/**
 * The controller responsible for displaing the registration page,
 * and for sending the users data away.
 */
user_controllers_module.controller('UserRegistrationCtrl',
  function($scope, $rootScope, $state, $stateParams, $q, $ionicNavBarDelegate, UserService, $ionicPlatform) {
    $ionicPlatform.ready(function() {
      if(typeof window.analytics !== 'undefined') { window.analytics.trackView("Account Registration");
      }
    });
  var page_from = $stateParams.pageFrom;

  var from_quiz = page_from == "quiz";
  var from_account = page_from == "account";

  $scope.$on('$ionicView.enter', function(){
      $ionicNavBarDelegate.showBackButton(false);
    });
  $scope.$on('$ionicView.leave', function(){
      $ionicNavBarDelegate.showBackButton(true);
    });

  if (from_account) {
    $scope.registration_message_paragraph = "This lets us reward you if you win, track your progress on the leaderboard, and tell you if you win a grand prize.";
  } else if (from_quiz) {
    $scope.registration_message_paragraph = "This lets us reward you if you win, track your progress on the leaderboard, and tell you if you win a grand prize.";
  }

  $scope.checkAge = function (input) {
    var age = moment(input);
    var now = moment();
    if (now.diff(age, 'years') < 18) {
      $scope.final_warnings = "Must be 18 or over to be eligible to win or appear on the leaderboard.";
    } else {
      $scope.final_warnings = null;
    }
  };

  $scope.submitInformation = function (formModel) {
    // formModel == userForm model.
    // $scope.form == the angular html class form.

    /**
     * Sets any invalid fields as told by the server as
     * invalid on the form.
     */
    var set_error_fields = function (server_response) {
      if (server_response.email) {
        $scope.form.email.$invalid = true;
      }

      if ((server_response.contestant || {}).leaderboard_name) {
        $scope.form.leaderboard_name.$invalid = true;
      }
    };
    /**
     * End Private Methods
     */

    if ($scope.form.$invalid) {
      $scope.field_errors = error_text($scope.form);
      return;
    }

    $scope.$emit('show-spinner');
    UserService.register(formModel).then(
      function (data) {
        $scope.$emit('hide-spinner');
        if (from_quiz) {
          $state.go('app.user_registration_thanks_quiz');
        } else {
          $state.go('app.user_registration_thanks_account');
        }
      },
      function (data) {
        $scope.$emit('hide-spinner');
        $scope.field_errors = error_text($scope.form, data.result);
        set_error_fields(data.result);
      });
  };
});

user_controllers_module.controller('UserRegistrationThanksFromQuizCtrl',
  function ($scope, UserService, resolvedUser, resolvedGameData) {

  $scope.user = resolvedUser.profile;

  $scope.playedGame = resolvedGameData;

  UserService.syncPlayHistory();
});

user_controllers_module.controller('UserSignInCtrl',
  function ($scope, $state, $stateParams, $ionicHistory, $ionicModal, $ionicPlatform, UserService) {
    $ionicPlatform.ready(function() {
      if(typeof window.analytics !== 'undefined') { window.analytics.trackView("Account SignIn");
      }
    });

  $scope.userForm = {};

  $scope.signIn = function (form) {
    $scope.field_errors = null;
    if ($scope.form.$invalid) {
      $scope.field_errors = "Fill Both Fields";
      return;
    }

    $scope.$emit('show-spinner');
    UserService.login(form.username, form.password).then(
      function (data) {
        // Advance to next screen.
        $ionicHistory.nextViewOptions({
            disableBack: true
        });
        $ionicHistory.clearHistory();

        UserService.syncProfile().finally(function () {
          $state.go("app.user_account");
        });
      },
      function (data) {
        $scope.$emit('hide-spinner');

        if (data.result.non_field_errors) {
          // Unable to login with credentials message.
          $scope.field_errors = data.result.non_field_errors[0];
        }
      }
    );
  };
});

user_controllers_module.controller('UserForgotPasswordCtrl', function ($scope, $rootScope, $http, $state, $ionicPlatform, UserService) {
  $ionicPlatform.ready(function() {
    if(typeof window.analytics !== 'undefined') { window.analytics.trackView("Account Lost Password");
    }
  });

  $scope.reset_password = false;

  $scope.resetPassword = function (email) {
    var reset_url = SERVER_ADDRESS + "user/forgot/";

    $scope.$broadcast('show-spinner');
    $http.post(reset_url, email)
      .then(function (data) {
        $scope.$broadcast('hide-spinner');
        $scope.reset_password = true;
      }, function (error_data) {
        $scope.$broadcast('hide-spinner');
        $scope.field_errors = "Unable to find that email";

        $scope.reset_password = false;
      });
  };
});

user_controllers_module.controller('UserCodesCtrl', function ($scope, $ionicPlatform, UserService, PagesService) {
  $ionicPlatform.ready(function() {
    if(typeof window.analytics !== 'undefined') { window.analytics.trackView("Prizes");
    }
  });

  $scope.checked_for_codes = false;
  $scope.history = [];

  PagesService.getAppText().then(
    function (success) {
      $scope.promotive_url = success.about_promotive_url;
    }
  );

  $scope.openPromotive = function () {
    cordova.InAppBrowser.open($scope.promotive_url, '_system', 'location=yes');
  };

  var get_codes = function () {
    UserService.syncPlayHistory().then().finally(function () {
      UserService.getPlayHistory().then(
        function (data) {
          $scope.history_type = data.type;
          $scope.history = data.history;
        },
        function (error_data) {
        }
      ).finally(function () {
        $scope.checked_for_codes = true;
        $scope.$broadcast('hide-spinner');
        $scope.$broadcast('scroll.refreshComplete');
      });
    });
  };

  var check_play_status = function () {
    UserService.ableToPlayQuizzes().then(function (status) {
      $scope.playable = status;
    });

    UserService.profile().then(function (data) {
      $scope.under18 = data.under18;
    }, function (data) {
      $scope.under18 = data.under18;
    });
  };

  $scope.$on('$ionicView.beforeEnter', function () {
    get_codes();
    check_play_status();
    if ($scope.history.length === 0 && $scope.checked_for_codes === false) {
      $scope.$broadcast('show-spinner');
    }
  });

  $scope.refreshCodes = function () {
    check_play_status();
    get_codes();
  };
});


/**
 * Determine what the error text for this form should be.
 * Really, just return the most prominent error. If no errors
 * exist, returns null.
 *
 * @return {string}
 */
var error_text = function (form, server_response) {
  if (server_response === null) {
    return "You need a signal to register.";
  } else if (form.$error.overThree) {
    return "Set a valid birthdate.";
  } else if (form.$error.required) {
    return "Fill all The fields.";
  } else if (form.$error.email) {
    return "Enter a valid email.";
  }

  if (server_response !== undefined) {
    if (server_response.email) {
      return "That email is already in use.";
    } else if ((server_response.contestant || {}).leaderboard_name) {
      return "That leaderboard name is already in use.";
    }
  }

  return null;
};
