var directives_module = angular.module('experticity.directives', ['experticity.quizDirectives', 'experticity.eventDirectives']);

/**
 * Given a prize-type of 'grand' or 'quiz', and a prize. prizeAnnounceText
 * will template either Grand Prize: or Prize: + the prize. Or blank if prize is blank.
 */
directives_module.directive('prizeAnnounceText', function () {
    return {
        scope: {
            prizeType: '=',
            prize: '=',
        },
        link: function (scope, elm, attrs) {
            var prize_empty = (scope.prize === null || scope.prize === undefined || scope.prize.length === 0);
            if (prize_empty || scope.can_give_prize === false) {
              scope.prize_msg = "";
              return scope.prize_msg;
            } else {
                if (scope.prizeType === "grand") {
                  scope.prize_msg = "Grand Prize: " + scope.prize;
                  return scope.prize_msg;
                } else {
                  scope.prize_msg = "Prize: " + scope.prize;
                  return scope.prize_msg;
                }
            }
        },
        template: "{{ prize_msg }}",
    };
});

/**
 * A Directive to create a hidden absolute div
 * beneath it's current. It then positions a material
 * design loader in the center.
 * Signal on and off with `show-spinner` and `hide-spinner`.
 */
directives_module.directive('materialLoader', function () {
  return {
    template: '<div class="material-loader Aligner" ng-class={"spinner-hidden":hide_spinner}>' +
                '<svg class="material-spinner" width="65px" height="65px" viewBox="0 0 66 66" xmlns="http://www.w3.org/2000/svg">' +
                  '<circle class="path" fill="none" stroke-width="6" stroke-linecap="round" cx="33" cy="33" r="30"></circle>' +
                '</svg>' +
              '</div><div ng-transclude></div>',
    link: function (scope, el, attr) {
      var loader = angular.element(el[0].getElementsByClassName('material-loader'));
      loader.css('min-height', el[0].offsetHeight + 'px');
      scope.hide_spinner = true;
      scope.$on('show-spinner', function () {
        scope.hide_spinner = false;
      });
      scope.$on('hide-spinner', function () {
        scope.hide_spinner = true;
      });
    },
    transclude: true,
  };
});

/**
 * A directive to get a click callback and go back
 * one level in the ionic view stack.
 */
directives_module.directive('goBack', function ($ionicHistory) {
  return {
    link: function (scope, el, attrs) {
      el.bind("click", function () {
        $ionicHistory.goBack();
      });
    },
  };
});

directives_module.directive('logoutButton', function ($state, $ionicHistory, UserService) {
  return {
    link: function (scope, el, attrs) {
      el.bind("click", function () {
        UserService.logout();
        $ionicHistory.nextViewOptions({
          disableBack: true
        });
        $state.go('app.user_registration', {'pageFrom': 'account'});
      });
    }
  };
});

directives_module.directive('stateNavBackButton', function($ionicHistory, $ionicViewSwitcher, $state) {
	return {
		restrict: 'A',
		priority: 10,
		link: function(scope, tElement, tAttrs) {
      tElement.bind('click', function () {
        if (tAttrs.parent.length !== 0) {
          $ionicHistory.nextViewOptions({
            disableBack: true,
            historyRoot: true
          });
          $ionicViewSwitcher.nextDirection('back');

          $state.go(tAttrs.parent);
        } else if ($ionicHistory.backView()) {
					$ionicHistory.goBack();
				}
      });
		}
	};
});

/**
* Over 3 directive. Ensures that a user is over 3 years old.
*/
directives_module.directive('over3', function () {
  return {
    require: 'ngModel',
    link: function (scope, el, attrs, ctrl) {
      ctrl.$validators.overThree = function(modelValue, viewValue) {
        var minBirthYear = new Date().getFullYear() - 3;
        var minAge = new Date();
        minAge.setFullYear(minBirthYear);
        var userAge = new Date(viewValue);

        return userAge < minAge;
      };
    }
  };
});

/**
 *	angular-count-to
 *
 *  The MIT License
    Copyright (c) 2013 Brian Matthews

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
 */
directives_module.directive('countTo', function ($timeout, $filter) {
  return {
      replace: false,
      scope: true,
      link: function (scope, element, attrs) {

          var e = element[0];
          var num, refreshInterval, duration, steps, step, countTo, value, increment;

          var calculate = function () {
              refreshInterval = 30;
              step = 0;
              scope.timoutId = null;
              countTo = parseInt(attrs.countTo) || 0;
              scope.value = parseInt(attrs.value, 10) || 0;
              duration = (parseFloat(attrs.duration) * 1000) || 0;

              steps = Math.ceil(duration / refreshInterval);
              increment = ((countTo - scope.value) / steps);
              num = scope.value;
          };

          var tick = function () {
              scope.timoutId = $timeout(function () {
                  num += increment;
                  step++;
                  if (step >= steps) {
                      $timeout.cancel(scope.timoutId);
                      num = countTo;
                      e.textContent = $filter('number')(countTo, 0);
                  } else {
                      e.textContent = $filter('number')(Math.round(num), 0);
                      tick();
                  }
              }, refreshInterval);

          };

          var start = function () {
              if (scope.timoutId) {
                  $timeout.cancel(scope.timoutId);
              }
              calculate();
              tick();
          };

          attrs.$observe('countTo', function (val) {
              if (val) {
                  start();
              }
          });

          attrs.$observe('value', function (val) {
              start();
          });

          return true;
      }
  };
});
