/**
 * Directives related to the event pages.
 */

 var event_directives_module = angular.module('experticity.eventDirectives', []);

/**
 * Directive: Expert-Tabs
 * Author: Kyle Swanson
 *
 * Set an active tab scope variable.
 */
 event_directives_module.directive('expertTabs', function ($stateParams) {
   var link = function(scope, el, attr) {
     if ($stateParams.tab.length !== 0) {
       scope.active_tab = $stateParams.tab;
     } else {
       scope.active_tab = "quizzes";
     }


     var parents = angular.element(el[0].querySelectorAll('[data-tab]'));
     parents.bind("click", function (event) {
       scope.active_tab = event.srcElement.dataset.tab;
     });
   };

   return {
     link: link,
   };
 });

/**
 * Logic for displaying a leaderboard. And refreshing it.
 *
 * Expects an eventId on the parent scope.
 */
 event_directives_module.directive('leaderboardList', function ($rootScope, $ionicPlatform, LeaderboardService) {
   var link = function (scope, el, attr) {
     $ionicPlatform.ready(function() {
       if(typeof window.analytics !== 'undefined') { window.analytics.trackView("Leaderboard List");
       }
     });

     scope.refreshLeaderboard = function () {
       loadLeaderboard();
     };

     var loadLeaderboard = function () {
       LeaderboardService.getLeaderboardForEvent(scope.eventId).then(
         function (data) {
           scope.scores = data;
         },
         function (error) {
           scope.scores = null;
         }
       ).finally(function () {
         scope.$broadcast('hide-spinner');
         scope.$broadcast('scroll.refreshComplete');
       });
     };

     scope.$broadcast('show-spinner');
     $rootScope.$on('$ionicView.enter',function () {
       loadLeaderboard();
     });
   };

   return {
     link: link,
   };
 });

 event_directives_module.directive('quizzesList', function ($rootScope, $state, $ionicHistory, $ionicPlatform, $ionicAnalytics, PlayService, UserService, QuizService) {
   var link = function (scope, el, attrs) {
     scope.refreshQuizzes = function () {
       $ionicPlatform.ready(function() {
         if(typeof window.analytics !== 'undefined') { window.analytics.trackView("Events Quizzes List");
         }
       });
       $ionicAnalytics.track('Events Quizzes List', {
         item_id: 'evql',
         item_name: 'Refresh',
       });

       QuizService.sync().then(function () {
         getQuizzes();
       });
     };

     var checkPlayStatus = function () {
       UserService.ableToPlayQuizzes().then(function (status) {
         scope.playable = status;
       });

       UserService.profile().then(function (result) {
         scope.under18 = result.under18;
       }, function (result) {
         scope.under18 = result.under18;
       });
     };
     checkPlayStatus();

     var getQuizzes = function () {
       QuizService.getEvent(scope.eventId).then(function (event) {
         loadQuizzes(event);
       }).finally(function () {
         scope.$broadcast('scroll.refreshComplete');
       });
     };
     getQuizzes();

     var loadQuizzes = function (e) {
       scope.event = e;

       UserService.scheduleNotifications(scope.event).then(function (quizzes) {});

       UserService.activeQuizzes(scope.event).then(function (quizzes) {
         scope.active_quizzes = quizzes;
       });

       UserService.futureQuizzes(scope.event).then(function (quizzes) {
         scope.future_quizzes = quizzes;
       });

       UserService.pastQuizzes(scope.event).then(function (quizzes) {
         scope.past_quizzes = quizzes;
       });
     };

     $rootScope.$on('$ionicView.enter',function () {
       UserService.syncPlayHistory();
       getQuizzes();
       checkPlayStatus();
     });

     scope.startQuiz = function (quiz) {
       $ionicPlatform.ready(function() {
         if(typeof window.analytics !== 'undefined') { window.analytics.trackView("Quiz Started"); }
       });
       $ionicAnalytics.track('Quiz Started', {
         item_id: 'qust',
         item_name: 'Start',
       });
       PlayService.startQuiz(quiz, scope.event);

       $state.go("quiz_question", {quizId: quiz.id, questionId: 0 });
     };
   };

   return {
     link: link,
   };
 });

 /*
  * To determine if the button in prizes list should send you to
  * event specifics or event list based on quiz availability.
  */
 quiz_directives_module.directive('viewEventButton', function ($state, $ionicViewSwitcher, $ionicHistory, UserService, QuizService) {

   var link = function (scope, element, attrs) {

     QuizService.getEvents().then(
       function (event_data) {
         active_events = event_data.data;
        //  Loop to get the active event id's into an array
         var list_of_event_ids = active_events.map(function (item, index) { return item.id; });

         var eventId = parseInt(attrs.viewEventButton)

         if (list_of_event_ids.indexOf(parseInt(attrs.viewEventButton)) !== -1) {
           element.bind("click", function () {
            //  $ionicHistory.clearHistory();
            //  $ionicHistory.clearCache();
            //  $ionicHistory.nextViewOptions({
            //    disableBack: true,
            //    disableAnimate: true
            //  });
             $state.go("app.event_quizzes", {eventId: eventId});
           });
           element.text("View Challenge");
         } else {
           element.bind("click", function () {
            //  $ionicHistory.clearHistory();
             $state.go("app.events");
           });
           element.text("Challenge Expired");
         }
       }
     );
   };

   return {
     link: link,
   };
 });
