/**
 * Directives related to the start, question, or end portion of quizzes.
 */

var quiz_directives_module = angular.module('experticity.quizDirectives', []);

quiz_directives_module.directive('countUpTimer', function ($rootScope, $timeout, $filter) {
  function link(scope, element, attrs) {
    var counter;
    var display_time = scope.totaltime;
    var paused = scope.pause;
    var question_elapsed_time = 0;

    function updateUi(t) {
      element.text($filter('number')(t, 2));

      // Call the method specififed.
      // scope.$applyAsync(scope.callback(question_elapsed_time));
    }

    element.on('$destroy', function () {
      $timeout.cancel(counter);
    });

    var STEP = 10;

    if (paused) {
      updateUi(display_time);
    } else {
      var count = function () {
        counter = $timeout(function () {
          question_elapsed_time = question_elapsed_time + (0.001 * STEP);
          display_time += (0.001 * STEP);
          updateUi(display_time);

          count();

        }, STEP);
      };

      count();

      $rootScope.$on('question-answered', function () {
        $timeout.cancel(counter);
        // Ensure the callback gets called.
        scope.$applyAsync(scope.callback(question_elapsed_time));
      });
    }
  }

  return {
    scope: {
      callback: '=',
      totaltime: '=',
      pause: '=',
    },
    link: link,
  };
});

quiz_directives_module.directive('quizButtonGroup', function () {
  function link(scope, element, attrs) {
    element[0].addEventListener("click", function (clicked_elm) {
      if (clicked_elm.toElement.localName == "a") {
        clicked_elm.toElement.className += " selected";
      }
    });
  }

  return {
    link: link
  };
});

quiz_directives_module.directive('zoomingMask', function ($rootScope) {
  function link(scope, element, attrs) {
    scope.mask_status = false;

    scope.$on('show-mask', function () {
      scope.mask_status = true;
    });

    var bolt_mask = angular.element(element[0].querySelector('.bolt-mask'));
    var mask_cell = angular.element(document.querySelector('.mask-cell'));

    bolt_mask.on('webkitTransitionEnd transitionend', function(event) {
      mask_cell.css('left','-100%');
      setObject(FIRST_RUN_HAPPENED, { has_run: true });
      scope.mask_status = false;
    });
  }

  return {
    scope: {
      play: '='
    },
    link: link,
    replace: 'true',
    template: '<div id="mask" class="mask-cell" ng-class="{\'run\': mask_status}">' +
                '<img class="logo" src="img/expert-logo.png">' +
                '<img class="bolt-mask" src="img/bolt-mask.svg">' +
                '<img class="pro-logo" src="img/powered-by.png">' +
              '</div>',
  };
});

quiz_directives_module.directive('blurOnSignal', function ($compile) {
  return {
    restrict: 'A',
    scope: {
      signal: '@',
    },
    link: function (scope, el, attrs) {
      var watcher = scope.$on(scope.signal, function () {
        new_el = el.clone();
        new_el.removeAttr('blur-on-signal');
        new_el.attr('frost', '');

        var fn = $compile(new_el)(scope);
        el.replaceWith(fn);

        watcher = null;
      });
    }
  };
});

/*
 * To determine if the button at the end of a quiz
 * should send you to the leaderboard or registration.
 */
quiz_directives_module.directive('quizCompletionButton', function ($state, $ionicViewSwitcher, UserService) {
  var link = function (scope, element, attrs) {

    UserService.profile().then(
      function (user_data) {
        if (user_data.profile !== undefined && user_data.profile.id !== null) {
          element.bind("click", function () {
            $state.go("app.event_quizzes", {eventId: scope.eventId, tab: 'leaderboard'});
          });
          element.text("View Leaderboard");
        } else {
          element.bind("click", function () {
            $state.go("app.user_registration", {pageFrom: "quiz"});
          });
          element.text("Continue");
        }
      }
    );
  };

  return {
    link: link,
    scope: {
      eventId: "@"
    }
  };
});
