/**
 *
 * Info.js
 *
 * Provides the different classes to hold info about events, quizzes, questions and choices.
 *
 */


/**
 * An event, it contains many Quizzes.
 */
var Event = function(parsed_json) {
  // console.log("Event Instantiated");

  this.id;
  this.name;
  this.start_time;
  this.end_time;
  this.location;
  this.description;
  this.grand_prize;
  this.event_image;
  this.quizzes = [];

  this.init = function (parsed_json) {
    this.importJson(parsed_json);
  };

  this.init(parsed_json);
};

Event.prototype = {
  constructor: Event,
  importJson: function(i) {
    this.id = i.id;
    this.name = i.name;
    this.start_time = i.start_time;
    this.end_time = i.end_time;
    this.location = i.location;
    this.description = i.description;
    this.grand_prize = i.grand_prize;
    this.event_image = i.event_image;

    // Parse the active quizzes.
    for (var x = 0; x < i.quizzes.length; x++) {
      var q = new Quiz(i.quizzes[x]);
      this.quizzes.push(q);
    }
  },
  /** A factory method, parses the json, and returns the event objects associated with it. */
  createEvents: function(json) {
    // If this is text, parse it to an object. if it's an object. Leave it alone.
    if (!json.event_list) {
      json = JSON.parse(json);
    }

    var events = [];
    var current_time = moment();

    for (var x = 0; x < json.event_list.length; x++) {
      var new_event = new Event(json.event_list[x]);

      // var split_start_date = new_event.start_time.split(/-| /);
      // var split_end_date = new_event.end_time.split(/-| /);
      // var split_start_time = split_start_date[3].split(":");
      // var split_start_seconds = split_start_time[2].split("+");
      // var split_end_time = split_end_date[3].split(":");
      // var split_end_seconds = split_start_time[2].split("+");
      //
      // var moment_start = moment({
      //    year: split_start_date[0],
      //    month: split_start_date[1] - 1,
      //    day: split_start_date[2],
      //    hour: split_start_time[0],
      //    minute: split_start_time[1],
      //    second: split_start_seconds[0],
      // });
      //
      // var moment_end = moment({
      //    year: split_end_date[0],
      //    month: split_end_date[1] - 1,
      //    day: split_end_date[2],
      //    hour: split_end_time[0],
      //    minute: split_end_time[1],
      //    second: split_end_seconds[0],
      // });
      // console.log(moment_start);
      // console.log(moment_end);
      // var after_start = current_time > moment_start;
      // var before_end = current_time < moment_end;


      //  if (after_start && before_end) {
      //    quizzes.push(quiz);
      //  }
      new_event.started = true;
      //  if (new_event.start_time >= )
      events.push(new_event);
    }

    return events;
  },
  /**
   * Return the date formmated to the apps design.
   * Ex: January 21-24, 2015.
   */
  getFormmatedDate: function () {
    var moment_start = moment(this.start_time);
    var moment_end = moment(this.end_time);

    var start_month = moment_start.format('MMMM');
    var end_month = moment_end.format('MMMM');


    var needs_to_display_both_months = start_month != end_month;
    if (needs_to_display_both_months) {
      // This event starts in a different month than when it ends.
      return start_month + " " + moment_start.format("D") +
            "-" + end_month + " " + moment_end.format("D") + ", " + moment_end.format("YYYY");
    } else {
      // Start month is the same as the end month.
      return start_month + " " + moment_start.format("D") + "-" +
            moment_end.format("D") + ", " + moment_end.format("YYYY");
    }
  },
  /**
   * Return true if the event should be playable now.
   * False if not.
   */
  isActive: function () {
    var moment_current = moment();

    var moment_start = moment(this.start_time);
    var moment_end = moment(this.end_time);

    var in_progress = moment_current > moment_start && moment_current < moment_end;

    if (in_progress) {
      return true;
    } else {
      return false;
    }
  },
  getImage: function () {
    if (this.event_image === null) {
      return "img/default-bg.jpg";
    } else {
      return this.event_image;
    }
  },
  getQuizzes: function () {
    var future = [];
    var active = [];
    var past = [];

    var current_time = moment();

    for (var x = 0; x < this.quizzes.length; x++) {
      var quiz = this.quizzes[x];

      var split_start_time = quiz.start_time.split(":");
      var split_end_time = quiz.end_time.split(":");

      var split_start_date = quiz.start_date.split(/-| /);
      var split_end_date = quiz.end_date.split(/-| /);

      var moment_start = moment({
        year: split_start_date[0],
        month: split_start_date[1] - 1,
        day: split_start_date[2],
        hour: split_start_time[0],
        minute: split_start_time[1],
        second: split_start_time[2],
      });

      var moment_end = moment({
        year: split_end_date[0],
        month: split_end_date[1] - 1,
        day: split_end_date[2],
        hour: split_end_time[0],
        minute: split_end_time[1],
        second: split_end_time[2],
      });

      var has_not_started = current_time < moment_start;
      var has_ended = current_time > moment_end;
      var in_progress = current_time > moment_start && current_time < moment_end;

      if (in_progress) {
        active.push(quiz);
      } else if (has_ended) {
        past.push(quiz);
      } else if (has_not_started) {
        future.push(quiz);
      }
    }

    return {
      future: future,
      active: active,
      past: past,
    };
  }
};

/** Represents a quiz. Pass in it's json information, and this will
convert it to a JS object. Use the associated methods to turn it back
into json */
var Quiz = function(parsed_json) {
  // console.log("Quiz Instantiated");

  this.id;
  this.name;
  this.start_date;
  this.end_date;
  this.start_time; // UTC
  this.end_time; // UTC
  this.prize;
  this.redemption_message;
  this.description;
  this.prize_threshold;
  this.double_down_prize_threshold;
  this.questions = []; // A list of Question objects.
  this.can_give_prize;


  this.able_to_double_down;
  this.double_down_question;

  this.__formmatedStartTime = null;

  this.init = function(parsed_json) {
    this.importJson(parsed_json);
  };

  this.init(parsed_json);
};

Quiz.prototype = {
  constructor: Quiz,
  importJson: function(parsed_json) {
    this.id = parsed_json.id;
    this.name = parsed_json.name;
    this.able_to_double_down = parsed_json.able_to_double_down;
    this.start_date = parsed_json.start_date;
    this.end_date = parsed_json.end_date;
    this.start_time = parsed_json.start_time;
    this.end_time = parsed_json.end_time;
    this.prize = parsed_json.prize;
    this.redemption_message = parsed_json.redemption_message;
    this.description = parsed_json.description;
    this.prize_threshold = parsed_json.prize_threshold;
    this.double_down_prize_threshold = parsed_json.double_down_prize_threshold;
    this.can_give_prize = parsed_json.can_give_prize;

    if (this.able_to_double_down && parsed_json.double_down_questions != undefined) {
          if (parsed_json.double_down_questions.length > 0) {
            this.double_down_question = new Question(parsed_json.double_down_questions[0]);
          }
    }


    for (var x = 0; x < parsed_json.questions.length; x++) {
      var q = new Question(parsed_json.questions[x]);
      this.questions.push(q);
    }
  },
  /** A factory, to create a quiz from parsed json */
  createQuiz: function(parsed_json) {
    var quiz = new Quiz(parsed_json);

    return quiz;
  },
  /**
   * Get the formmated time.
   */
  getFormmatedStartTime: function () {
    // Premature Optimization is the Root of Most Programming Errors.
    if (this.__formmatedStartTime !== null) {
        return this.__formmatedStartTime;
    }

    var splits = this.start_time.split(":");

    var moment_start = moment({
        hour: splits[0],
        minute: splits[1],
        second: splits[2],
      });

    this.__formmatedStartTime = moment_start.format("h:mm a");

    return this.__formmatedStartTime;
  },
  getFormmatedEndTime: function () {
    var splits = this.end_time.split(":");

    var moment_end = moment({
        hour: splits[0],
        minute: splits[1],
        second: splits[2],
      });

    return moment_end.format("h:mm a");
  },
  getFormmatedStartDate: function () {
    var splits = this.start_date.split(/-| /);

    var moment_date = moment({
      year: splits[0],
      // MomentJS has Jan set to 0
      month: splits[1] - 1,
      day: splits[2],
    });

    return moment_date.format("MMM Do");
  },
  getFormmatedEndDate: function () {
    var splits = this.end_date.split(/-| /);

    var moment_date = moment({
      year: splits[0],
      // MomentJS has Jan set to 0
      month: splits[1] - 1,
      day: splits[2],
    });

    return moment_date.format("MMM Do");
  },
  hasPrize: function () {
    return this.prize !== "";
  },
  /**
   * Return what prize a person will get if they
   * win this quiz.
   *
   * @return {string} the prize awarded.
   */
  getPrize: function () {
    if (this.can_give_prize) {
      return this.prize;
    } else {
      return "A Place on the Leaderboard";
    }
  },
  /**
   * @return {boolean} if there are prizes left for this quiz.
   */
  hasPrizesLeft: function () {
    return this.can_give_prize;
  },
  /**
   * Get the max number of points for a quiz.
   * If there is a double down question. Multiply that by 2.
   *
   * @return {integer} Summation of quiz points
   */
  getMaxPoints: function () {
    var question_points = this.questions.reduce(function (prev, cur, index, array) {
      if (typeof cur.max_points==='number' && (cur.max_points%1)===0) {
        return prev + parseInt(cur.max_points);
      } else {
        return prev;
      }
    }, 0);

    if (this.able_to_double_down) {
      return question_points * 2;
    } else {
      return question_points;
    }
  }
};

/**
 * A set of choices. Contains info on the quiz, caches the image. And returns Json.
 */
var Question = function(parsed_json) {
  // console.log("Question Instantiated");

  /**
   * TODO: Look at the image in the questions, then cache it somewhere.
   */
  this.id;
  this.text;
  this.image;
  this.max_points;
  this.max_time;

  this.false_choices = []; // A list of Choice objects.
  this.correct_choice; // A single Choice object.

  this.init = function(parsed_json) {
    this.importJson(parsed_json);
  };

  this.init(parsed_json);
};

Question.prototype = {
  constructor: Question,
  checkAnswer: function (choice) {
    return this.checkAnswerById(choice.id);
  },
  checkAnswerById: function (id) {
    return (id == this.correct_choice.id);
  },
  /** Returns all of the choices (correct and false), randomized in order. */
  getChoices: function () {
    var choices = this.false_choices;
    choices.push(this.correct_choice);

    choices = shuffleArray(choices);
    return choices;
  },
  importJson: function (parsed_json) {
    this.id = parsed_json.id;
    this.text = parsed_json.text;
    this.image = parsed_json.image;
    this.max_points = parsed_json.max_points;
    this.max_time = parsed_json.max_time;

    this.correct_choice = new Choice(parsed_json.correct_choice);
    for (var x = 0; x < parsed_json.false_choices.length; x++) {
      var c = new Choice(parsed_json.false_choices[x]);
      this.false_choices.push(c);
    }
  },
  hasImage: function () {
    return this.image != "";
  }
};

/**
 * A single choice in a quiz.
 */
var Choice = function(parsed_json) {
  // console.log("Choice Instantiated");

  this.id;
  this.text;

  this.init = function(parsed_json) {
    this.importJson(parsed_json);
  }

  this.init(parsed_json);
};

Choice.prototype = {
  constructor: Choice,
  importJson: function(parsed_json) {
    this.id = parsed_json.id;
    this.text = parsed_json.text;
  },
};
