/**
 *
 * State.js
 *
 * Provides the different classes to maintain information about a users progress.
 *
 * PlayedGames store info about a game played, as well as all their answers in that game.
 *
 */


/**
 * Methods anad classes to keep track of the state of a game.
 */
var SAVED_PLAYED_GAMES = "7_SAVED_PLAYED_GAMES";

var PlayedGame = function (parsed_json) {
  this.quiz; // The quiz object.
  this.play_time; //The time this quiz was played at.
  this.answers = []; //A collection of answer objects
  this.event;
  this.completed; // If the user completed this quiz. Incomplete should not
                          // be playable again, but also shouldnt go to the server.
  this.sent_to_server;

  this._double_down_answer = null; //A Private variable to speed up a few methods.

  this.init = function (parsed_json) {
    this.importJson(parsed_json);
    this.completed = true; //True unless otherwise
  };

  this.init(parsed_json);
};

PlayedGame.prototype = {
  constructor: PlayedGame,
  // Return all the answer objects.
  getAnswers: function () {
    return this.answers;
  },
  // Return a list of correct answers.
  getCorrectAnswers: function () {
    var correct = [];

    for (var x = 0; x < this.answers.length; x++) {
      var answer = this.answers[x];
      if (answer.isCorrect()) {
        correct.push(correct);
      }
    }

    return correct;
  },
  // Return a list of incorrect answers.
  getIncorrectAnswers: function () {

  },
  //Check if they won prize
  //True if they earned one.
  //False if not.
  canGetPrize: function () {
    //   First check if there is a prize. You can't get a non existent prize ya know.
    if (this.quiz.prize === "") {
        return false;
    }

    // Second check if they doubled down. That can disqualify you.
    if (this.gameWasDoubledDown()) {
      if (this.getDoubleDownAnswer().isCorrect() === false) {
        // console.log('>>>>>> Incorrect Double Down')
        return false;
      }
    }

    var total = this.getAnswers().length;
    var correct = this.getCorrectAnswers().length;

    var percentage = correct / total;
    // console.log('percentage',percentage);
    // Check this related to the threshold.
    return percentage >= this.quiz.prize_threshold;
  },
  // Calculate the points earned for this game.
  getPointsEarned: function () {
    var multiplier = 1;

    var was_doubled_down = this.gameWasDoubledDown();
    if (was_doubled_down) {
      var double_down_is_correct = this.getDoubleDownAnswer().isCorrect();
      if (double_down_is_correct) {
        multiplier = 2;
      } else {
        return 0;
      }
    }

    var total_points = 0;

    for (var x = 0; x < this.answers.length; x++) {
      var answer = this.answers[x];

      if (!isNaN(answer.points())) {
        total_points = total_points + answer.points();
      }
    }
    return total_points * multiplier;
  },
  // Return the play time in seconds.
  getPlayTime: function () {
    var pt = 0;

    for (var x = 0; x < this.answers.length; x++) {
      pt += this.answers[x].elapsed_time;
    }

    return pt;
  },
  // Return the play time in a readable format. 2 Min 38 Sec.
  // Actually 02:38
  getFormmatedPlayTime: function () {
    var total_seconds = this.getPlayTime();
    var seconds = total_seconds % 60;
    var minutes = (total_seconds - seconds) / 60;

    return pad(minutes, 2) + ":" + pad(Math.round(seconds), 2);
  },
  /**
   * Save the quiz to a list of local storage things.
   */
  save: function () {
    var history = this.getSaved();
    history.push(this);
    setObject(SAVED_PLAYED_GAMES, {games: history});
  },
  /**
   * Return all the saved quizzes.
   */
  getSaved: function () {
    var saved = [];
    var saved_played_games = getObject(SAVED_PLAYED_GAMES);
    if (saved_played_games.games === undefined) {
      saved_played_games.games = [];
    }
    for (var x = 0; x < saved_played_games.games.length; x++) {
      var game = saved_played_games.games[x];
      if (game.quiz !== undefined) {
        // console.log("getSaved", game);
        var pg = PlayedGame.prototype.createPlayedGame(game.quiz, game.play_time, game.answers, game.event, game.sent_to_server);
        pg.completed = game.completed;
        saved.push(pg);
      }
    }

    return saved;
  },
  /**
   * Mayda, this'll clear everything.
   */
  clearAllPlayedGames: function () {
    setObject(SAVED_PLAYED_GAMES, {});
  },
  // Calculate the total points earned. For every game the user has played.
  getTotalPoints: function () {
    var saved_games = this.getSaved();
    var total_points = 0;

    for (var x = 0; x < saved_games.length; x++) {
      var game = saved_games[x];

      total_points += game.getPointsEarned();
    }

    return total_points;
  },
  getTotalPointsFormmated: function () {
    var points = this.getTotalPoints();
    var rounded_points = Math.round(points);


    return rounded_points.toLocaleString('en-US');
  },
  /**
   * Return true or false if the game was doubled down.
   */
  gameWasDoubledDown: function () {
    // The private var is filled. Keep things fast and say true.
    if (this._double_down_answer !== null) {
      return true;
    } else {
      // console.log("else gameWasDoubledDown()");
      for (var x = 0; x < this.answers.length; x++) {
        var answer = this.answers[x];
        if (answer.double_down_question) {
          this._double_down_answer = answer;
          return true;
        }
      }

      return false;
    }
  },
  /**
   * Return the question that the user tried to double
   * down on.
   */
  getDoubleDownAnswer: function () {
    if (this._double_down_answer !== null) {
      return this._double_down_answer;
    } else {
      this.gameWasDoubledDown();
      return this._double_down_answer;
    }
  },
  /**
   * Import the parsed json, into this object.
   */
  importJson: function (parsed_json) {
    this.quiz = parsed_json.quiz; // Convert this to a quiz object.
    this.play_time = parsed_json.play_time;
    this.answers = parsed_json.answers; // Convert these to answer objects.
    this.event = parsed_json.event;
    this.sent_to_server = parsed_json.sent_to_server;
  },
  /**
   * Export this PlayedGame to json. So that it can be beamed up to the server.
   */
  exportJson: function () {
    // The formmat the server expects.
    var answers = this.getAnswers();
    // To speed this up, we could eliminate correct.
    // and build the number below.
    var correct = this.getCorrectAnswers();

    var questions_array = [];
    for (var x = 0; x < answers.length; x++) {
      var answer = answers[x];

      var d = {
        question: answer.question.id,
        choice: answer.choice.id,
        answered_correctly: answer.isCorrect(),
        points_awarded: answer.points(),
        time_taken: answer.elapsed_time,
        was_a_double_down_question: answer.double_down_question,
      };

      questions_array.push(d);
    }

    var data = {
      quiz: this.quiz.id,
      event: this.event.id,
      answers_correct: correct.length,
      answer_possible: answers.length,
      time_to_complete: this.getPlayTime(),
      won_a_prize: this.quiz.can_give_prize && this.canGetPrize(),
      questions: questions_array
    };

    return data;
  },
  /**
   * Create a PlayedGame object. A factory.
   */
  createPlayedGame: function (quiz, play_time, answers, event, sent_to_server) {
    var parsed_json = {};

    parsed_json.quiz = Quiz.prototype.createQuiz(quiz);
    parsed_json.play_time = play_time;

    if (sent_to_server === undefined) {
      sent_to_server = false;
    }

    parsed_json.sent_to_server = sent_to_server;

    parsed_json.answers = [];
    for (var x = 0; x < answers.length; x++) {
      var answer = answers[x];
      parsed_json.answers.push(
        Answer.prototype.createAnswer(answer.question, answer.choice, answer.elapsed_time, answer.double_down_question)
      );
    }

    parsed_json.event = event;

    return new PlayedGame(parsed_json);
  }
};

var Answer = function (parsed_json) {
  this.question; //Question object
  this.choice; //Choice object
  this.elapsed_time; //How long it took to answer the question
  this.double_down_question;

  this.init = function (parsed_json) {
    this.importJson(parsed_json);
  };

  this.init(parsed_json);
};

Answer.prototype = {
  constructor: Answer,
  isCorrect: function () {
    return this.question.checkAnswer(this.choice);
  },
  /**
   * Return the number of points this answer earned.
   */
  points: function () {
    if (this.isCorrect()) {
      return Math.round(this.question.max_points / this.elapsed_time);
    } else {
      return 0;
    }

  },
  importJson: function (parsed_json) {
    this.question = new Question(parsed_json.question);
    this.choice = new Choice(parsed_json.choice);
    this.elapsed_time = parsed_json.elapsed_time;
    this.double_down_question = parsed_json.double_down_question;
  },
  createAnswer: function (question, choice, elapsed_time, double_down_question) {
    var parsed_json = {};
    parsed_json.question = question;
    parsed_json.choice = choice;
    parsed_json.elapsed_time = elapsed_time;
    parsed_json.double_down_question = double_down_question;

    return new Answer(parsed_json);
  },
};
