var EVENT_DATA_SERVICE = 'event_data_service_8675309';
var PAGE_DATA_SERVICE = 'page_data_service_434';

angular.module('experticity.dataService', ['ionic'])

/**
 * QuizService
 *
 * Responsible for syncing events from the server and their
 * quizzes/questions.
 *
 * The method `sync` will get new things from the server.
 * All public methods use promises.
 *
 * At initialization, `sync` is called automatically.
 *
 * Public Methods:
 * - sync
 * - getEvents
 * - getEvent(int eventId)
 */
.service('QuizService', function ($q, $http) {
  console.log("QuizService");
  var EVENT_SERVER_ADDRESS = function () { return SERVER_ADDRESS + "events/?rand=" + parseInt(Math.random()*99999999); };

  var event_data_cache = {'data':[]};

  /**
   * Private Methods
   */

  var eventsFromServer = function () {
    var dfd = $q.defer();

    $http.get(EVENT_SERVER_ADDRESS()).
      success(function (data, status, headers, config) {
        setObject(EVENT_DATA_SERVICE, data.event_list);
        return dfd.resolve({'data':data, 'status':status});
      }).
      error(function (data, status, headers, config) {
        return dfd.resolve({'data':data, 'status':status});
      });

    return dfd.promise;
  };

  var eventsFromLocalStorage = function () {
    console.log("eventsFromLocalStorage");
    var dfd = $q.defer();

    (function () {
      var events = getObject(EVENT_DATA_SERVICE, '[]');

      if (events.length !== 0) {
        dfd.resolve({'data':events, 'status':10341});
      } else {
        dfd.reject({'data':events, 'status':10341});
      }
    })();

    return dfd.promise;
  };

  /**
   * End Private Methods
   */


  eventsFromServer();

  return {
    /**
     * Using simple logic, sync the most recent results from the server.
     *
     * @return {Promise} The result of this sync. Either
     *                     - {result:'server'} -> Data connection allowed a sync
     *                     - {result:'local'} -> No data connection to sync
     */
    sync: function () {
      var dfd = $q.defer();

      eventsFromServer().then(
        function (success) {
          console.log("Clear the variable caches -> " + event_data_cache);
          event_data_cache = {'data':[]};

          dfd.resolve({'result': 'server'});
        },
        function (error) {
          dfd.resolve({'result': 'local'});
        }
      );

      return dfd.promise;
    },
    /**
     * Return the events stored on the device. Usually this will be instant.
     * But in the event of no sync yet. It may take a moment.
     *
     * @return {Promise} A list of events.
     *                     - {events: [array of events]}
     */
    getEvents: function () {
      var dfd = $q.defer();

      if (event_data_cache.data.length > 0) {
        dfd.resolve(event_data_cache);
      } else {
        eventsFromLocalStorage().then(
          function (success) {
            event_data_cache = success;
            event_data_cache.data = Event.prototype.createEvents({event_list:success.data});
            event_data_cache.data.reverse();

            dfd.resolve(event_data_cache);
          },
          function (error) {
            dfd.reject(error);
          }
        );
      }

      return dfd.promise;
    },
    /**
     * Return a single event stored on the device. Usually this will be instant.
     * But in the event of no sync yet. It may take a moment.
     *
     * @param  {Integer} eventId the Id of the event you want.
     * @return {Promise}         A single event.
     *                             - {event: {}}
     */
    getEvent: function (eventId) {
      var dfd = $q.defer();

      this.getEvents().then(
        function (events) {
          for (var i = 0; i < events.data.length; i++) {
            var event = events.data[i];
            if (event.id === parseInt(eventId)) {
              dfd.resolve(event);
              return;
            }
          }
        }
      );

      return dfd.promise;
    }
  };
})

/**
 * Service to fetch leaderboard scores for an event.
 */
.service('LeaderboardService', function ($q, $http, UserService) {
  return {
    getLeaderboardForEvent: function (eventId) {
      var dfd = $q.defer();

      var score_url = SERVER_ADDRESS + "quiz/leaderboard/" + eventId + "/";
      var config = UserService.getAuthHeaders();


      $http.get(score_url, config).
        success(function (data, status, headers, config) {
          dfd.resolve(data);
        }).
        error(function (data, status, headers, config) {
          dfd.reject(data);
        });

      return dfd.promise;
    },
  };
})

.service('PagesService', function ($q, $http) {
  var getData = function () {
    return getObject(PAGE_DATA_SERVICE, '{}');
  };

  var setData = function (d) {
    setObject(PAGE_DATA_SERVICE, d);
  };

  return {
    downloadData: function () {
      var dfd = $q.defer();

      $http.get(SERVER_ADDRESS + "get/pages/", { cache: true }).
        success(function(data, status, headers, config) {
          setData(data);

          dfd.resolve({'response': data});
        }).
        error(function (data, status, headers, config) {
          console.error("Error trying to retrieve pages from server");

          data = { "app_text": {
              "welcome_screen_tag_line": "Think you're an expert?",
              "welcome_screen_text": "Complete a series of quizzes for a chance to win great prizes you can redeem during each event. Be the highest ranked expert at the end of each event to win the grand prize.",
              "double_down_tag_line": "Answer one extra question.",
              "double_down_text": "Get it right: Win the quiz prize - plus double your points from this quiz towards the grand prize leaderboard. \n\r Get it wrong: Lose the points and prize from this quiz.",
              "about_promotive_url": "https://www.promotive.com/",
          },
          "pages": [] };
          if (getData().data === undefined) {
            setData(PAGE_DATA_SERVICE, data);
          }

          dfd.reject({'response': "Unable to resolve data"});
        });

      return dfd.promise;
    },
    getPages: function () {
      var dfd = $q.defer();

      (function (ctx, promise) {
        console.log(getData());
        if (getData().pages === undefined) {
          ctx.downloadData().then().finally(function () {
            promise.resolve(getData().pages);
          });
        } else {
          promise.resolve(getData().pages);
        }
      }) (this, dfd);

      return dfd.promise;
    },
    getAppText: function () {
      var dfd = $q.defer();

      (function (ctx, promise) {
        if (getData().app_text === undefined) {
          ctx.downloadData().then().finally(function () {
            promise.resolve(getData().app_text);
          });
        } else {
          promise.resolve(getData().app_text);
        }
      }) (this, dfd);

      return dfd.promise;
    }
  };
});
