angular.module('experticity.playService', ['ionic'])

/**
 * Maintain the state across playing a quiz.
 */
.service('PlayService', function($q, $http, UserService) {
  var quiz;
  var event;
  var current_question;
  var current_question_index = 0;
  var answers = [];
  var start_time;
  var play_time = 0;

  var final_play_data;

  function __reset_vars () {
    quiz = null;
    current_question = null;
    current_question_index = 0;
    answers = [];
    start_time = null;
    play_time = 0;

    final_play_data = null;
  }
  return {
    startQuiz: function (q, e) {
      __reset_vars();

      quiz = q;
      start_time = Date.now();
      answers = [];
      event = e;
      current_question = quiz.questions[current_question_index];

      // Catches a user that quits early by saving their game as played.
      start_play_data = PlayedGame.prototype.createPlayedGame(quiz, start_time, answers, event);
      start_play_data.completed = false;
      start_play_data.save();
      UserService.addPlayedGame(start_play_data);

      return this.quizStatus();
    },
    /**
     * Return statistics about the quiz. Current question #, number of questions. Etc.
     */
    quizStatus: function () {
      var out_of_questions = current_question_index == quiz.questions.length;

      return {
        quiz: quiz,
        event: event,
        current_question: current_question,
        current_question_position: current_question_index + 1,
        play_time: play_time,
        quiz_is_over: out_of_questions,
      };
    },
    /**
     * Answer a question, return true or false based on the choice.
     */
    answerQuestion: function (answer_choice, elapsed_time, double_down) {
      // Add this answer to the list of answer objects.
      var ans = Answer.prototype.createAnswer(current_question, answer_choice, elapsed_time, double_down);
      answers.push(ans);

      var correct_answer = current_question.checkAnswer(answer_choice);
      //Advance the play time
      play_time += elapsed_time;

      if (double_down === false) {
        // Advance the current question.
        current_question_index++;
        current_question = quiz.questions[current_question_index];
      }

      if (correct_answer) {
        return true;
      } else {
        return false;
      }
    },
    /**
     * Used for double downing.
     */
    setDoubleDownQuestion: function (q) {
      current_question = q;
    },
    /**
     * Returns the percentage of the anwered questions so far
     * that are correct. Ex: 0.5
     */
    percentageCorrect: function () {
      var correct = 0;

      for (var x = 0; x < answers.length; x++) {
        if (answers[x].isCorrect()) correct += 1;
      }

      return correct/answers.length;
    },
    /**
     * Create a "PlayedGame" object and save it.
     */
    endQuiz: function (user_doubled_down, user_won_double_down) {
      var dfd = $q.defer();

      (function (ctx) {
        final_play_data = PlayedGame.prototype.createPlayedGame(quiz, start_time, answers, event);
        final_play_data.save();
        var games = UserService.getPlayedGames();
        new_game_id = final_play_data.quiz.id
        // This is a loop/pop to take off the most recently added played game.
        // This was added in the startQuiz func to catch cheating by quiting early.
        for (var x = 0; x < games.length; x++) {
          if (games[x].quiz.id == new_game_id) {
            games.pop();
            setObject(PLAYED_GAME_DATA, games);
            break;
          };
        };
        dfd.resolve(final_play_data);
        UserService.addPlayedGame(final_play_data);
        var after_game = UserService.getPlayedGames();
      })(this);

      return dfd.promise;
    },

    finalQuizData: function () {
      return final_play_data;
    },
  };
});
